package ch.rakudave.jnetmap.plugins;

import java.awt.Frame;

import org.apache.commons.collections15.Factory;

import ch.rakudave.jnetmap.model.Map;
import ch.rakudave.jnetmap.model.device.DeviceEvent;

/**
 * A special MapListener designed for Notifiers. 
 * Only notifies DeviceEvents of Type STATUS_CHANGED and INTERFACE_STATUS_CHANGED
 * Any fields of implementing classes that are not serializable with 
 * xstream MUST use the @XStreamOmitField-annotation or saving a map will fail...
 * 
 * @author rakudave
 *
 */
public interface Notifier extends Plugin, Factory<Notifier>{
		
	/**
	 * Notifies the plugin when the status of a device or an interface has changed
	 * @param e the associated DeviceEvent
	 */
	public void statusChanged(DeviceEvent e, Map m);
	
	/**
	 * Shows a properties-window for this instance of the plugin, 
	 * for example to set the email-address that should be notified of the change
	 * @param owner to be displayed on top of
	 * @param isSetup true the first time after instance creation
	 */
	public void showPropertiesWindow(Frame owner, boolean isSetup);
	
	/**
	 * @return Descriptive name of this instance, such as "Notifies some@one.com"
	 */
	public String getName();
}
