package ch.rakudave.jnetmap.view.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Vector;

import javax.swing.DefaultListCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import ch.rakudave.jnetmap.model.Connection;
import ch.rakudave.jnetmap.model.device.Device.Type;
import ch.rakudave.jnetmap.model.factories.ConnectionFactory;
import ch.rakudave.jnetmap.model.factories.DeviceFactory;
import ch.rakudave.jnetmap.util.Icons;
import ch.rakudave.jnetmap.util.Settings;
import ch.rakudave.jnetmap.util.Tuple;
import ch.rakudave.jnetmap.util.logging.Logger;


@SuppressWarnings({ "unchecked", "rawtypes", "serial" })
public class EditPanel extends JPanel implements ListSelectionListener, ActionListener, KeyListener, MouseListener {
	private JList icons;
	private JComboBox connectionType;
	private JTextField bandwidth;
	private Vector<Tuple<String, ImageIcon>> types;
	
	/**
	 * Set the default device-type, connection-type and -speed for the factories
	 * @param owner
	 */
	public EditPanel(final Frame owner) {
		setLayout(new BorderLayout(5, 5));
		types = new Vector<Tuple<String, ImageIcon>>();
		for (Type t : Type.values()) {
			if (t != Type.Other) types.add(new Tuple<String, ImageIcon>(t.toString(), Icons.getCisco(t.toString().toLowerCase())));
		}
		for (int i = 0; i < Settings.getInt("type.custom.count", 0); i++) {
			types.add(new Tuple<String, ImageIcon>(Type.Other.toString(), Icons.fromBase64(Settings.get("type.custom."+i, ""))));
		}
		icons = new JList(types);
			icons.setCellRenderer(new IconCellRenderer());
			icons.setSelectedValue(Type.Workstation, true);
			icons.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
			icons.setLayoutOrientation(JList.HORIZONTAL_WRAP);
			icons.setVisibleRowCount(-1);
			icons.addListSelectionListener(this);
			icons.setBackground(new Color(getBackground().getRGB()));
			icons.addMouseListener(this);
		JPanel bottom = new JPanel(new GridLayout(0, 2, 5, 5));
			connectionType = new JComboBox(Connection.Type.values());
				connectionType.setSelectedItem(Connection.Type.Ethernet);
				connectionType.addActionListener(this);
			JPanel inner = new JPanel(new GridLayout(0, 2, 5, 5));
				bandwidth = new JTextField("100");
					bandwidth.addKeyListener(this);
				inner.add(bandwidth);
				inner.add(new JLabel("Mb/s"));
			bottom.add(connectionType);
			bottom.add(inner);
		add(new JScrollPane(icons), BorderLayout.CENTER);
		add(bottom, BorderLayout.SOUTH);
		//TODO not an optimal solution, clogs up preferences-file. consider using separate file and class 
		/*JButton add = new JButton(Lang.get("action.add"), Icons.get("add"));
			add.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					File f = SwingHelper.openDialog(EditPanel.this, new FileNameExtensionFilter("PNG Image", "png"));
					if (f != null) {
						String base64 = Icons.getBase64(f);
						int count = Settings.getInt("type.custom.count", 0);
						Settings.put("type.custom."+count, base64);
						Settings.put("type.custom.count", count+1);
						Logger.debug("Adding custom device type: "+base64);
						types.add(new Tuple<String, ImageIcon>(Type.Other.toString(), Icons.fromBase64(base64)));
						DeviceFactory.setType(Type.Other); DeviceFactory.setIcon(base64);
						icons.setSelectedIndex(types.size()-1);
						TabPanel.getEditingModeSetter().actionPerformed(null);
						EditPanel.this.repaint();
					}
				}
			});
		add(add, BorderLayout.NORTH);*/
	}
	
	private class IconCellRenderer extends DefaultListCellRenderer {
		@Override
		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			JLabel label = (JLabel) super.getListCellRendererComponent(list,
					value, index, isSelected, cellHasFocus);
			if (value == null) return label;
				label.setText(null);
				label.setToolTipText(((Tuple<String, ImageIcon>) value).getFirst());
				label.setIcon(((Tuple<String, ImageIcon>) value).getSecond());
				label.setHorizontalAlignment(CENTER);
				label.setVerticalAlignment(CENTER);
			return label;
		}
	}

	@Override
	public void valueChanged(ListSelectionEvent e) {
		Tuple<String, ImageIcon> tuple = (Tuple<String, ImageIcon>)icons.getSelectedValue();
		DeviceFactory.setType(Type.valueOf(tuple.getFirst()));
		if (Type.Other.toString().equals(tuple.getFirst())) DeviceFactory.setIcon(tuple.getSecond().getDescription());
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		ConnectionFactory.setType((Connection.Type) connectionType.getSelectedItem());
	}

	@Override public void keyTyped(KeyEvent e) {}
	@Override public void keyPressed(KeyEvent e) {}
	@Override
	public void keyReleased(KeyEvent e) {
		try {
			double d = Double.valueOf(bandwidth.getText());
			ConnectionFactory.setSpeed(d);
			Logger.trace("Default bandwidth set to "+d);
		} catch (Exception e2) {
			Logger.trace("Unable to convert to double: "+bandwidth.getText(), e2);
			bandwidth.setText("100");
		}
	}

	@Override public void mouseClicked(MouseEvent e) {}
	@Override public void mousePressed(MouseEvent e) {
		TabPanel.getEditingModeSetter().actionPerformed(null);
	}
	@Override public void mouseReleased(MouseEvent e) {}
	@Override public void mouseEntered(MouseEvent e) {}
	@Override public void mouseExited(MouseEvent e) {}
}
